; Similar to buildgalsed but gives fluxes in a set of filters - used when fitting to photometry.

function buildgalphot, filters, p, wav=wav, flambda=flambda, freq=freq, fnu=fnu, binning=binning, resolution=resolution, hist=hist, Lbol=Lbol, Lir=Lir, nonebular=nonebular
  ; Calculate fluxes in various filters - used for SED fitting.

  ; freq and fnu will give you the output spectrum when you're done.

  ppos = ppos()
  
  z = p[ppos.z]
  metallicity = p[ppos.metal]
  sfr = p[ppos.sfr]
  tav = p[ppos.tav]
  age = [p[ppos.age], p[ppos.age2]]
  texp = p[ppos.tau]
  mass = p[ppos.mass]
  embedfrac = p[ppos.fembed]
  av = p[ppos.av]
  embedav = p[ppos.avembed]
  dustlaws = extmap()
  dustlaw = dustlaws[p[ppos.dustlaw]]
  rv = p[ppos.rv]
  Tdust = p[ppos.Tdust]
  q = p[ppos.q]
  gasmetal = p[ppos.gasmetal]
  tsmooth = p[ppos.tsmooth]
  
  ;if n_elements(binning) eq 0 then binning = 3.
  ;if n_elements(resolution) eq 0 then resolution = 3.
  
  buildgalspec, freq, fnu, z=z, $
                 sfr=sfr, tav=tav, age=age, model=model, texp=texp, tsmooth=tsmooth, metallicity=metallicity, mass=mass, $
                 embedfrac=embedfrac, av=av, embedav=embedav, dustlaw=dustlaw, rv=rv, $
                 Tdust=Tdust, gasmetal=gasmetal, q=q, $
                 binning=binning, resolution=resolution, $
                 Lbol=Lbol, Lir=Lir, hist=hist, $
                 lines=lines, linefluxes=linefluxes, nonebular=nonebular

  wav = 3e9/freq
  flambda = fnu*1d-29*(freq*1e9)/wav

  nf = n_elements(filters)
  isline = intarr(nf)
  for f = 0, nf-1 do begin
     if strlen(filters[f]) lt 2 then continue
     if strmid(filters[f],0,2) eq 'H_' or strmid(filters[f],0,1) eq '[' then begin
       isline[f] = 1
     endif
  endfor

  wherestd = where(isline eq 0, nstd, complement=whereline)
  if nstd gt 0 then begin
     stdfilters = filters[wherestd]
     stdfilterfluxes = synphot(wav, flambda, stdfilters, xunit='Ang', yunit='flambda')
  endif
  nlin = nf-nstd
  if nlin gt 0 then begin
     linefilters = filters[whereline]
     linefilterfluxes = fltarr(nlin)
     for l = 0, n_elements(linefilters)-1 do begin
        match = where(linefilters[l] eq lines, ct)
        if ct gt 0 then linefilterfluxes[l] = linefluxes[match]
        if ct eq 0 and n_elements(lines) gt 1 then print, 'Unrecognized line filter'
     endfor
  endif
  
  fluxes = fltarr(nf)
  if nstd gt 0 then fluxes[wherestd] = stdfilterfluxes
  if nlin gt 0 then fluxes[whereline] = linefilterfluxes

  ;print, p
  ;print, fluxes

  return, fluxes

end

