function eprsame, val, same, ndec
  ; Prints out a number in exponential notation with the same exponent as another number.
  ; Useful for printing an uncertainty with the same power as the measurement, e.g.
  ; IDL> print, eprsame(7.5e+7, 1.23e+9)
  ;    0.075e+9

  if n_elements(ndec) eq 0 then ndec = 4
  if val eq 0.0 then val = val+1e10-1e10 ; -0's are annoying

  samepower = floor(alog10(same))
  valf = val / 10.0^samepower
  valfpower = floor(alog10(valf))

  npredec = valfpower+1 > 1
  npostdec = ndec+1-npredec

  return, fpr(valf, strtrim(fix(npredec),2)+'.'+strtrim(fix(npostdec),2)) + 'e' + string(samepower,format='(I+03)')

end
