; Special plotter for SEDs.

pro sedplot, xinit, yinit, xrange=xrange, yrange=yrange, xunit=xunit, yunit=yunit, xtitle=xtitle, ytitle=ytitle, x2unit=x2unit, y2unit=y2unit, x2title=x2title, y2title=y2title, z=z, stripe=stripe, scolor=scolor, ssize=ssize, srange=srange, snudge=snudge, orientation=orientation, xtickformat=xtickformat, ytickformat=ytickformat, x2tickformat=x2tickformat, y2tickformat=y2tickformat, righttick=righttick, charsize=charsize

   common sedvars, sedxunit, sedyunit, sedz, lum2flux, $
       xstr, xoff, xcla, xtit,   ystr, yoff, ycla, ytit

   ; options:

   ; x:
   ;  lambda -
   ;    Angstroms   (ang)
   ;    microns     (um)
   ;    nanometers  (nm)
   ;    centimeters (cm)
   ;    meters      (m)
   ;  nu -
   ;    Hz          (Hz)
   ;    MHz         (MHz)
   ;    GHz         (GHz)
   ;  rest_ appended on the above indicates rest-frame quantity.
   ;
   ; y:
   ;  fnu -
   ;    erg/cm^s/s/Hz (fnu)
   ;    jansky      (Jy)
   ;    millijansky (mJy)
   ;    microjansky (uJy)
   ;    nanojansky  (nJy)
   ;  mag -
   ;    AB mag      (AB)
   ;  flambda -
   ;    erg/cm^2/s/cm (flambda/cgs)
   ;    erg/cm^2/s/Ang (flambda/ang)
   ;  nu*fnu or lambda*flambda
   ;    erg/cm^2/s   (nufnu)
   ;  Lnu -
   ;    erg/s/Hz     (Lnu)
   ;  Llambda -
   ;    erg/s/Ang    (Llambda)
   ;  nu*Lnu
   ;    erg/s        (nuLnu)
   ;    Lsun/s       (nuLsolnu)

   if n_elements(stripe) eq 0 then stripe = 'fnu'
   if n_elements(scolor) eq 0 then scolor = 16
   if n_elements(ssize) eq 0 then ssize = 0.8
   if n_elements(srange) eq 0 then srange = [-!values.f_infinity,!values.f_infinity]
   if n_elements(snudge) eq 0 then snudge = [0,0]
   if n_elements(z) eq 0 then z = 0.
   sedxunit = xunit
   sedyunit = yunit
    
   if n_elements(x2unit) gt 0 then if x2unit eq '' then delvarx, x2unit
   if n_elements(y2unit) gt 0 then if y2unit eq '' then delvarx, y2unit

   if n_elements(x2unit) gt 0 then if n_elements(x2tickformat) eq 0                    then x2tickformat='expaxis'
   if n_elements(y2unit) gt 0 then if n_elements(y2tickformat) eq 0 and y2unit ne 'AB' then y2tickformat='expaxis'

   ang = '!19'+string(197B)+'!17'

   xstr = ['ang','nm','um','mm','cm','m']
   xoff = [1d-8, 1d-7, 1d-4, 1d-1, 1., 1d2]
   xtit = 'Wavelength (' + [ang,'nm','!4m!17m','mm','cm','m'] + ')'
   xcla = ['wav', 'wav', 'wav', 'wav', 'wav', 'wav']

   xstr = [xstr, 'rest_'+xstr]
   xoff = [xoff,  xoff*(1+z)]
   xtit = [xtit, 'Rest '+xtit]
   xcla = [xcla, xcla]

   xstr = [xstr,   ['Hz','MHz','GHz']]
   xoff = [xoff,   [1.,  1e6,  1e9]]
   xtit = [xtit,  'Frequency (' + ['Hz','MHz','GHz']+')']
   xcla = [xcla, ['freq', 'freq', 'freq']]

   xstr = [xstr,    'rest_'+['Hz','MHz','GHz']]
   xoff = [xoff,     xoff/(1+z)]
   xtit = [xtit,  'Rest '+xtit]
   xcla = [xcla, ['freq', 'freq', 'freq']]
   
   ystr = ['fnu', 'Jy', 'mJy', 'uJy', 'nJy']
   yoff = [1.,   1d-23, 1d-26, 1d-29, 1d-32]
   ytit = 'Flux F!D!4n!17!N (' + ['erg/s/cm!U2!N/Hz','Jy','mJy','!4m!17Jy','nJy'] + ')'
   ycla = ['fnu', 'fnu','fnu', 'fnu', 'fnu']

   ystr = [ystr,   'AB']
   yoff = [yoff,    1.]
   ytit = [ytit, 'AB magnitude']
   ycla = [ycla, 'mag']

   ystr = [ystr,  ['flambda/cgs', 'flambda/ang']]
   yoff = [yoff,    1.,            1e8          ]
   ytit = [ytit, 'Flux F!D!4n!17!N (' + ['erg/s/cm!U3!N','erg/s/cm!U2!N/'+ang] + ')']
   ycla = [ycla,['flambda','flambda']]

   ystr = [ystr, 'nufnu']
   yoff = [yoff,    1.]
   ytit = [ytit, 'Specific flux !4n!17F!D!4n!17!N (erg/s/cm!U2!N)']
   ycla = [ycla, 'nufnu']

   ystr = [ystr, 'Lnu']
   yoff = [yoff,    1.]
   ytit = [ytit, 'Specific luminosity L!D!4n!17!D (erg/s/Hz)']
   ycla = [ycla, 'Lnu']

   ystr = [ystr,    'Llambda']
   yoff = [yoff,    1.]
   ytit = [ytit, 'Specific luminosity L!D!4n!17!D (erg/s/'+ang+')']
   ycla = [ycla, 'Llambda']

   ystr = [ystr,  ['nuLnu', 'nuLsolnu']]
   yoff = [yoff,  [1.,  3.9d33]]
   ytit = [ytit, 'Specific luminosity !4n!17L!D!4n!17!N ('+['erg/s','L!Dsol!N']+')']
   ycla = [ycla, ['nuLnu', 'nuLnu']]

   if n_elements(z) gt 0 then begin
      lum2flux = 1.d / (4.d * !pi * (3.086d24*lumdist(z,/silent,H0=H0, k=k, Omega_M=Omega_M, Lambda0=Lambda0, q0=q0))^2 )
   endif

   if yunit eq 'AB' then ylog = 0 else ylog = 1

   xstyle = 1  ; 8 = left only, 4 = neither
   ystyle = 1
   if n_elements(x2unit) gt 0 or n_elements(x2title) gt 0 then xstyle += 8
   if n_elements(y2unit) gt 0 or n_elements(y2title) gt 0 or n_elements(righttick) then ystyle += 8
   if n_elements(xtitle) eq 0 then begin
      xtitle = xtit[where(xunit eq xstr)]
   endif
   if n_elements(x2title) eq 0 and n_elements(x2unit) gt 0 then begin
      x2title = xtit[where(x2unit eq xstr)]
   endif
   if n_elements(ytitle) eq 0 then begin
      ytitle = ytit[where(yunit eq ystr)]
   endif
   if n_elements(y2title) eq 0 and n_elements(y2unit) gt 0 then begin
      y2title = ytit[where(y2unit eq ystr)]
   endif

   plot, [0], [0], xrange=xrange, yrange=yrange, /xlog, ylog=ylog, xstyle=xstyle, ystyle=ystyle, xtitle=xtitle, ytitle=ytitle, title='!17', $
         xtickformat=xtickformat, ytickformat=ytickformat, xminor=9, charsize=charsize
 ;, xstyle=8+4+1, ystyle=8+4+1, xtickformat='(A1)', ytickformat='(A1)', title='!17'

   ; should allow arbitrary striping...

   ;if yunit eq 'flambda/cgs' or yunit eq 'flambda/ang' or yunit eq 'nufnu' then begin
      if n_elements(orientation) eq 0 then orientation = -48  ; someday calculate this from the scale and plot axis ratio
      if stripe eq 'mag' then begin
         mags = findgen(40)
         for m = 0, n_elements(mags)-1 do begin
            convert, xrange, [mags[m], mags[m]], xxx, yline, xunit=xunit, yunit='AB'
            oplot, xrange, yline, color=scolor
            if yline[0] lt yrange[1] then begin
               capx = xrange[0]*(xrange[1]/xrange[0])^(0.02)
               capy = yline[0]*(yrange[1]/yrange[0])^(-0.01)
            endif else begin
               capy = yrange[1]*(yrange[1]/yrange[0])^(-0.05)
               capx = xrange[0] * (yline[0]/capy) * (xrange[1]/xrange[0])^(0.02)
            endelse
            str = clip(fix(m))+ ' mag'
            if mags[m] ge min(srange) and mags[m] le max(srange) then $
            if capx lt xrange[1]*(xrange[1]/xrange[0])^(-0.06) and capy gt yrange[0]*(yrange[1]/yrange[0])^(0.06)  then $ 
               xyouts, capx*(1+snudge[0]), capy+snudge[1], str, orientation=orientation, color=scolor, charsize=ssize
         endfor
      endif
      if stripe eq 'fnu' then begin
         fluxes = 10.d^(dindgen(25)-10)
         for m = 0, n_elements(fluxes)-1 do begin
            convert, xrange, [fluxes[m], fluxes[m]], xxx, yline, xunit=xunit, yunit='uJy'
            oplot, xrange, yline, color=scolor
            if fluxes[m] le 1e-3                      then str = expaxisiso(0,0,fluxes[m])+' !4m!17Jy'
            if fluxes[m] ge 1e-3 and fluxes[m] lt 1e3 then str = clipzero(fluxes[m])+' !4m!17Jy'
            if fluxes[m] ge 1e3 and fluxes[m] lt 1e6  then str = clipzero(fluxes[m]/1000.)+' mJy'
            if fluxes[m] ge 1e6                       then str = clipzero(fluxes[m]/1e6)+' Jy'
            ; left, then top mode
            ;if yline[0] lt yrange[1]*(yrange[1]/yrange[0])^(-0.02) then begin  ; left
            ;   capx = xrange[0]*(xrange[1]/xrange[0])^(0.02)
            ;   capy = yline[0]*(yrange[1]/yrange[0])^(-0.01)
            ;endif else begin                                                   ; top
            ;   capy = yrange[1]*(yrange[1]/yrange[0])^(-0.05) ; order is important
            ;   capx = xrange[0] * (yline[0]/capy) * (xrange[1]/xrange[0])^(0.02)
            ;endelse
            ; bottom, then right mode
            if yline[1] lt yrange[0] then begin ; bottom
               capy = yrange[0]*(yrange[1]/yrange[0])^(0.2)
               capx = xrange[1] * (yline[1]/capy) * (xrange[1]/xrange[0])^(0.07)
            endif else begin                                                 ; right
               capx = xrange[0]*(xrange[1]/xrange[0])^(0.85)
               capy = yline[1]*(yrange[1]/yrange[0])^(0.2)
            endelse 

            if fluxes[m] ge min(srange) and fluxes[m] le max(srange) then $
            if capx lt xrange[1]*(xrange[1]/xrange[0])^(-0.09) and capy gt yrange[0]*(yrange[1]/yrange[0])^(0.06)  then $
               xyouts, capx*(1+snudge[0]), capy*(1+snudge[1]), str, orientation=orientation, color=scolor, charsize=ssize
         endfor
      endif
      if stripe eq 'nuLsolnu' then begin
         lums = 10.^(dindgen(16))
         for m = 0, n_elements(lums)-1 do begin
            convert, xrange, [lums[m], lums[m]], xxx, yline, xunit=xunit, yunit='nuLsolnu'
            oplot, xrange, yline, color=scolor
            str = expaxisiso(0,0,lums[m])+' L!Do!N'
            if yline[1] gt min(yrange) and yline[1] lt max(yrange) then begin
                capx = xrange[1]*(1.0d*xrange[1]/xrange[0])^(-0.08)
                capy = yline[1]*(1.0d*yrange[1]/yrange[0])^(-0.05)
            endif else begin
                if yunit ne 'AB' then capy = yrange[1]*(1.0d*yrange[1]/yrange[0])^(-0.1) $
                                 else capy = yrange[1] + 1.2
                if yunit ne 'AB' then capx = xrange[1] * (1.0d*capy/yline[1]) * (1.0d*xrange[1]/xrange[0])^(-0.02) $
                                 else capx = xrange[1] * (1.0d*2.51^(-capy)/2.51^(-yline[1])) * (1.0d*xrange[1]/xrange[0])^(-0.02)
            endelse
            if capx gt xrange[0] and capx lt xrange[1]  then $
                xyouts, capx, capy, str, orientation=orientation, color=scolor, charsize=ssize
         endfor
      endif
   ;endif

   if n_elements(x2unit) then begin
      convert, xrange, x2range, xunit=xunit, x2unit=x2unit
      ;print, xrange ; I think there is a bug here
      ;print, x2range
      axis, xaxis=1, /xlog,  xrange=x2range, /xsty, xtitle=x2title, xtickformat=x2tickformat, xminor=9  ; used to be x2range/(1+z) for some reason
      ;, xtickv=[0.1,0.2,0.4,0.6,1.0,2.0,4.0], xminor=0, xticks=5
      ;axis, xaxis=1, /xlog,  xrange=xrange/(1+z), /xsty, xtickformat='(A1)'
   endif
   if n_elements(y2unit) then begin
      convert, [xrange[1], xrange[1]], yrange, x2rangey, y2range, xunit=xunit, x2unit=x2unit, yunit=yunit, y2unit=y2unit
      if y2unit eq 'AB' then y2log = 0 else y2log = 1
       
      axis, yaxis=1, yrange=y2range, ylog=y2log, ystyle=1, yminor=9, ytitle=y2title, ytickformat=y2tickformat
      if y2range[0] gt 1d+20 then axis, yaxis=1, yrange=y2range/1d+30, ylog=y2log, ystyle=1, yminor=9, xtickformat='(A1)'
        ; IDL forgets tickmarks if yrange is very large for some reason

   endif

   if n_elements(righttick) then axis, yaxis=1, yticks=1, yminor=1, ytickformat='(A1)', yticklen=0

;;   axis, yaxis=0,  yrange=yrange, /ylog,  ystyle=1,  yminor=9, yticks=7, ytickv=10.^[0,1,2,3,4,5,6,7,8], /noerase, ytickformat='(A1)'
;;   axis, yaxis=0,  yrange=1d+9*1d-29*yrange,  /ylog, ystyle=1,yminor=9, yticks=7, ytickv=1d+9*1d-29*10.d^[1,2,3,4,5,6,7,8], /noerase, ytitle=ytitle, ytickformat=ytickformat


     if n_Elements(xinit) gt 0 and n_elements(yinit) gt 0 then begin
       addsedplot, xinit, yinit, xunit=xunit, yunit=yunit ;, color=color
     endif


end


pro addsedplot,  x, y,  xunit=xunit, yunit=yunit, style=style, color=color, thick=thick

   convert, x, y, plotx, ploty, xunit=xunit, yunit=yunit
 
   oplot, plotx, ploty, linestyle=style, color=color, thick=thick
end

pro addsedpolyfill, x, y,  xunit=xunit, yunit=yunit, color=color

   convert, x, y, plotx, ploty, xunit=xunit, yunit=yunit
 
   polyfill, plotx, ploty, color=color
end

pro addsedmegaplot, x, y, xunit=xunit, yunit=yunit, $
 xerr=xerr, xmerr=xmerr, xperr=xperr, yerr=yerr, ymerr=ymerr, yperr=yperr, psym=psym, $
 size=size, fill=fill, color=color, limit=limit, xlimit=xlimit, hatlength=hatlength, $
 errcolor=errcolor

   convert, x, y, plotx, ploty, xunit=xunit, yunit=yunit

   n = n_elements(x)
   ;print, xunit, yunit
   ;for i = 0, n-1 do print, x[i], y[i], plotx[i], ploty[i]

   if n_elements(xerr) gt 0  then begin
        convert, x+xerr,  plotxpluserr, xunit=xunit   ; should do this more carefully
        plotxerr = plotxpluserr-plotx
   endif
   if n_elements(xmerr) gt 0  then begin
        convert, x-xmerr,  plotxminuserr, xunit=xunit
        plotxmerr = -plotxminusserr+plotx
   endif
   if n_elements(xperr) gt 0  then begin
        convert, x+xperr,  plotxpluserr, xunit=xunit
        plotxperr = plotxpluserr-plotx
   endif
   if n_elements(yerr) gt 0  then begin
        convert, x, y+yerr,  repeatx, plotypluserr,  xunit=xunit, yunit=yunit
        convert, x, y-yerr,  repeatx, plotyminuserr,  xunit=xunit, yunit=yunit
        ;plotyerr = abs(plotypluserr-ploty)
        plotyperr = plotypluserr-ploty
        plotymerr = -plotyminuserr+ploty
        for i = 0, n_elements(plotyperr)-1 do begin
          if plotyperr[i] lt 0 and plotymerr[i] lt 0 then begin
             plotyperr[i] = -(-plotyminuserr[i]+ploty[i])
             plotymerr[i] = -(plotypluserr[i]-ploty[i])
          endif
        endfor
   endif
   if n_elements(ymerr) gt 0  then begin
        convert, x, y-ymerr, repeatx, plotyminuserr, xunit=xunit, yunit=yunit
        plotymerr = abs(-plotyminuserr+ploty)
   endif
   if n_elements(yperr) gt 0  then begin
        convert, x, y+yperr, repeatx, plotypluserr, xunit=xunit, yunit=yunit
        plotyperr = abs(plotypluserr-ploty)
   endif
   ;if n_elements(xmerr) gt 0 then convert, x+xmerr, pxmerr, xunit=xunit
   ;if n_elements(xperr) gt 0 then convert, x+xperr, plotxperr, xunit=xunit
   ;if n_elements(yerr) gt 0  then convert, x, y+yerr,  repeatx, plotyerr,  xunit=xunit, yunit=yunit
   ;if n_elements(ymerr) gt 0 then convert, x, y+ymerr, repeatx, plotymerr, xunit=xunit, yunit=yunit
   ;if n_elements(yperr) gt 0 then convert, x, y+yeerr, repeatx, plotyperr, xunit=xunit, yunit=yunit

   ;if n_elements(plotyperr) gt 0 or n_elements(plotymerr) then print, ploty
   ;if n_elements(plotyperr) gt 0 then print, 'p', plotyperr
   ;if n_elements(plotymerr) gt 0 then print, 'm', plotymerr
   
   megaplot, plotx, ploty, /noerase, xerr=plotxerr, xmerr=plotxmerr, xperr=plotxperr, yerr=plotyerr, ymerr=plotymerr, yperr=plotyperr, psym=psym, size=size, fill=fill, color=color, limit=limit, xlimit=xlimit, hatlength=hatlength, errcolor=errcolor
end


pro convert, x, y, plotx, ploty, xunit=xunit, yunit=yunit, x2unit=x2unit, y2unit=y2unit
   ; somewhat confusing reuse of 'x2unit' here -> conversion-to unit, not right axis (which is meaningless here)

   common sedvars, sedxunit, sedyunit, sedz, lum2flux, $
       xstr, xoff, xcla, xtit,   ystr, yoff, ycla, ytit

   skipy = 0
   if n_elements(y) eq 0 and n_elements(yunit) eq 0 then begin
       skipy = 1
   endif
   if n_elements(x2unit) eq 0 and n_elements(sedxunit) gt 0 then x2unit = sedxunit
   if n_elements(y2unit) eq 0 and n_elements(sedyunit) gt 0 then y2unit = sedyunit


   pxin   =  (where(xunit eq xstr,  ctxin)) [0]
   pxout  =  (where(x2unit eq xstr, ctxout)) [0]
   if skipy eq 0 then begin
     pyin  = (where(yunit eq ystr, ctyin)) [0]
     pyout = (where(y2unit eq ystr, ctyout)) [0]
   endif

   if pxin lt 0 then print, 'Unrecognized unit: ', xunit
   if pxout lt 0 then print, 'Unrecognized unit: ', x2unit
   if skipy eq 0 then begin
     if pyin lt 0 then print, 'Unrecognized unit: ', yunit
     if pyout lt 0 then print, 'Unrecognized unit: ', y2unit
   endif

   c = 2.998d10

   if xcla[pxin] eq 'wav' then begin
      wavcm = x * xoff[pxin]
      freqHz = c / wavcm
   endif
   if xcla[pxin] eq 'freq' then begin
      freqHz = x * xoff[pxin]
      wavcm = c / freqHz
   endif

   if xcla[pxout] eq 'wav'  then plotx = wavcm /  xoff[pxout]
   if xcla[pxout] eq 'freq' then plotx = freqHz / xoff[pxout]

   if skipy eq 0 then begin
     if ycla[pyin] eq 'fnu'     then fnu = y * yoff[pyin]
     if ycla[pyin] eq 'mag'     then fnu = 1d-29 * mag2ujy(y, 'AB')
     if ycla[pyin] eq 'flambda' then fnu = y * yoff[pyin] * (wavcm / freqHz)
     if ycla[pyin] eq 'nufnu'   then fnu = y * yoff[pyin] / freqHz
     if ycla[pyin] eq 'Lnu'     then fnu = y * yoff[pyin] * lum2flux
     if ycla[pyin] eq 'Llambda' then fnu = y * yoff[pyin] * (wavcm / freqHz) * lum2flux
     if ycla[pyin] eq 'nuLnu'   then fnu = y * yoff[pyin] / freqHz * lum2flux
     if ycla[pyout] eq 'fnu'     then ploty = fnu / yoff[pyout]
     if ycla[pyout] eq 'mag'     then ploty = ujy2mag(fnu/1d-29, 'AB')
     if ycla[pyout] eq 'flambda' then ploty = fnu / (yoff[pyout] * (wavcm / freqHz))
     if ycla[pyout] eq 'nufnu'   then ploty = fnu * freqHz / (yoff[pyout])
     if ycla[pyout] eq 'Lnu'     then ploty = fnu / (yoff[pyout] * lum2flux)
     if ycla[pyout] eq 'Llambda' then ploty = fnu / (yoff[pyout] * (wavcm / freqHz) * lum2flux)
     if ycla[pyout] eq 'nuLnu'   then ploty = fnu * freqHz / (yoff[pyout] * lum2flux)
   endif

   if skipy then y = plotx

end



pro openplot, filename=filename, xs=xs, ys=ys, colors=colors, invcolor=invcolor, font=font
   set_plot, 'ps'
   device, filename=filename, /encapsulated
   !p.font = 0
   device, /symbol, font_index = 4
   if n_elements(font) eq 0 then font = 'times'
   if font eq 'times' then begin
     device, /times, font_index = 17
     device, /times, /bold, font_index = 18
   endif
   if font eq 'helvetica' then begin
     device, /helvetica, font_index = 17
     device, /helvetica, /bold, font_index = 18
   endif
   device, /isolatin1, font_index = 19
   device, /inches, xsize=xs, ysize=ys
   ;device, /inches, yoffset=4.5+(5.5-ys)
   device, /color
   if n_elements(colors) eq 0 then begin
      colors = transpose([[0,0,0],$
          [255,90,245],$        ;UVW2    1
          [200,0,180],$         ;UVM2
          [120,0,105],$         ;UVW1
          [130,40,180],$        ;U u
          [0,0,255],$           ;B       5
          [0,200,200],$         ;g
          [0,220,0],$           ;V
          [255,128,0],$         ;R
          [255,0,0],$           ;I
          [200,0,120],$         ;z       10
          [225,90,160],$         ;J
          [205,180,0],$         ;H
          [235,184,133], $      ;K       13
          [255,255,0], $        ;yellow    14
          [128,128,128], $
          [172,172,172], $
          [204,204,204], $
          [255,255,255]])
   endif
   if keyword_set(invcolor) then colors[1:17,*] = 255 - colors[1:17,*]
   tvlct, colors

end


pro testsed
   !p.position = [0.13, 0.13, 0.89, 0.87]
   openplot, filename='test.ps', xs=5, ys=3.5

   sedplot, xrange=[0.1, 10], yrange=[1e-30, 1e-28], xunit='um', yunit='fnu', z=1.0, x2unit='rest_um', y2unit='uJy'

   wav_um = [0.2, 0.4, 0.8, 1.6, 3.2, 6.4]
   freq_Hz = (3d14 / wav_um)
   fnu_cgs = 1e-29 * wav_um

   fnu_uJy = fnu_cgs / 1e-29
   nufnu   = fnu_cgs * freq_Hz

   addsedplot, wav_um, fnu_cgs, xunit='um', yunit='fnu', thick=60, color=15
   addsedplot, freq_Hz,fnu_cgs, xunit='Hz', yunit='fnu', thick=50, color=13
   addsedplot, wav_um, fnu_uJy, xunit='um', yunit='uJy', thick=40, color=12
   addsedplot, wav_um, nufnu,   xunit='um', yunit='nufnu', thick=30, color=10

   addsedmegaplot, wav_um, fnu_cgs, xunit='um', yunit='fnu', xerr=wav_um*0.5, yerr=fnu_cgs*0.5

   psclose
end


