@buildgalaxy

pro fitgalaxyflux, grb=grb, z=z, age=age, mass=mass, metallicity=metallicity, embedav=embedav, dustlaw=dustlaw, rv=rv, Tdust=Tdust

  if n_elements(z) eq 0 then z = 2.
  if n_elements(sfr) eq 0 then sfr = 100.
  if n_elements(age) eq 0 then age = 100e+6
  if n_elements(metallicity) eq 0 then metallicity = 0.4
  if n_elements(embedfrac) eq 0 then embedfrac = 0.
  if n_elements(av) eq 0 then av = 0.
  if n_elements(embedav) eq 0 then embedav = 50.
  if n_elements(Tdust) eq 0 then Tdust = 40.
  
  obs = readhostphot('~/research/darkesthost/hostphotometry.dat')

  gobs = obs[where(obs.grb eq grb)]
  fluxerr = mag2ujy(gobs.abmag-gobs.emag,'AB')-mag2ujy(gobs.abmag,'AB')

  n_p = 13
  parinfo = replicate({value:-1.0, fixed:1, limited:[0,0], limits:[0.,0.]}, n_p)
  ; note: fixed by default

  ; redshift
  parinfo[0].value = z

  ; sfr - typically free as it governs normalization
  parinfo[1].value = sfr
  parinfo[1].fixed = 0
  parinfo[1].limited = [1, 0]
  parinfo[1].limits = [0, 10000.]
  
  ; age
  parinfo[2].value = age
  parinfo[2].limited = [1, 0]
  parinfo[2].limits = [0, 15e+9]

  ; model - no change necessary

  ; texp - no change necessary

  ; metallicity
  parinfo[5].value = metallicity

  ; mass is currently uniquely determined by age and SFR.  we will free it later.
  parinfo[6].value = -1
  ;parinfo[6].limited = [1, 0]
  ;parinfo[6].limits = [0., 1e+14]

  ; embedfrac
  parinfo[7].value = embedfrac
  parinfo[7].limited = [1, 1]
  parinfo[7].limits = [0., 1.]

  ; av - typically free as it is a somewhat artificial quantity
  parinfo[8].value = 0.5
  parinfo[8].fixed = 0
  parinfo[8].limited = [1,0]
  parinfo[8].limits = [0., 100.]

  ; embedav
  parinfo[9].value = embedav
  parinfo[9].limited = [1, 0]
  parinfo[9].limits = [0., 100.]

  ; dustlaw - no change necessary

  ; rv - no change necessary

  ; Tdust
  parinfo[12].value = embedav
  parinfo[12].limited = [1, 0]
  parinfo[12].limits = [0., 100.]

  print, gobs.flux
  print, gobs.fluxerr
  
  gwav = filtwv(gobs.filter, fwhm=fwhm)
  window, 0
  megaplot, gwav, gobs.flux, xerr=fwhm, yerr=gobs.fluxerr, /xlog, /ylog

  sol = mpfitfun('galaxyfiltflux', gobs.filter, gobs.flux, gobs.fluxerr, parinfo=parinfo,$
                  perr=iperr, bestnorm=ichisq, dof=dof, yfit=iyfit, /quiet)

  megaplot, gwav, gobs.flux, xerr=fwhm, yerr=gobs.fluxerr, /xlog, /ylog

  f = galaxyfiltflux(gobs.filter, sol)  ; just for the plot...

  print, ichisq , '/', dof
  print, sol



end

