@buildgalaxy
@buildgalphot
@sedplot

function parstring, name, abbrev, val, dval, fixed, err, perr, merr
   str = ''
   str +=  clip(name, 27)
   str += rclip(abbrev, 6) + ' ='
   if dval gt 1e4 then str += '   '+epr(dval,3) else str += fpr(dval,4.3)+'    '
   if fixed and perr le 0 then begin
      if val gt 0 or abbrev eq 'Av' then str += ' (fixed)' else str += ' (derived)'
   endif else begin
      if perr eq 0 or finite(perr) eq 0 then begin
         str += ' +/- '
         if err gt 1e4 then str += eprsame(err,dval,3) else str += clip(fpr(err,4.3))
      endif else begin
         str += ' -'
         if dval gt 1e4 then str += eprsame(merr,dval,3) else str += clip(fpr(merr,4.3))
         str += ' +'
         if dval gt 1e4 then str += eprsame(perr,dval,3) else str += clip(fpr(perr,4.3))
      endelse
   endelse

   return, str
end

function parstringtable, val, dval, fixed, err, perr, merr, format=format, nchar=nchar
   str = ''
   if n_elements(nchar) eq 0 then nchar = 8
   if n_Elements(format) eq 0 then format = 8.2
   str +=  clip(fpr(dval,format),nchar)
   if fixed and perr le 0 then begin
       if val gt 0 then str += '^F' else str += '\pm??$
   endif else begin
     if perr gt 0 or merr gt 0 then begin
        str += '(+'+clip(fpr(perr,format),nchar)+','+clip(fpr(-merr,format),nchar)+')'
     endif else begin
        str += '\pm'+fpr(err,format)
     endelse
   endelse
   return, str
end

function fullsol, sol
   fullsol = sol
   ppos = ppos()
   if n_elements(sol) eq 1 then begin 
      return, fltarr(ppos.npar) + !values.f_nan
   endif
   if sol[ppos.age] eq -1 then begin
      if sol[ppos.tau] gt 0 then begin
         fullsol[ppos.age] = sol[ppos.tau] * alog(sol[ppos.mass] / (sol[ppos.sfr]*sol[ppos.tau]) + 1) 
      endif
      if sol[ppos.tau] eq -1 then begin ; infinite texp
         fullsol[ppos.age] = sol[ppos.mass] / sol[ppos.sfr]
      endif
   endif
   if sol[ppos.sfr] eq -1 then begin
     if sol[ppos.tau] eq -1 then begin
        fullsol[ppos.sfr] = sol[ppos.mass] / sol[ppos.age]
     endif
   endif
   ; here would be a age -> sfr
   ; here would be a age -> mass
   ; also some complicated two-population stuff
   ; neeed to regenerate if you care about this
   ;if sol[ppos.l] eq -1 then begin
   ;   fullsol[ppos.l] = lbol
   ;endif

   return, fullsol
end

pro galsedfit, galaxy, filter, flux, fluxerr, z=z, ebv=ebv, photometryfile=photometryfile, longfile=longfile, modelfile=modelfile, loud=loud, replot=replot, xrange=xrange,yrange=yrange, xunit=xunit, yunit=yunit, inset=inset, stripe=stripe, orient=orient, x2unit=x2unit, y2unit=y2unit, ntrials=ntrials, alsoplot=alsoplot, residualsinfo=residualsinfo, minage=minage, minunc=minunc, addunc=addunc, resolution=resolution

;  KEY INPUTS:
;  galaxy                : name of the galaxy to fit.  (If blank, fit everything.)
;  filter, flux, fluxerr : arrays of filter strings, fluxes and flux uncertainties in uJy, corrected for foreground dust (optional)
;  photometryfile        : photometry file
;  longfile              : photometry file for submm/radio measurements.  Semi-deprecated, generally this should be omitted.
;  modelfile             : file of initial guesses.
;  z, ebv                : redshift and Galactic dust column, if no modelfile is given.

;  OPTIONS:
;  loud                  : Flag, set to print out the mpfit iterations to monitor fit progress.
;  replot                : Just redo the plot with the stored values - don't actually fit again.
;  xrange                : two-element array of minimum and maximum x values
;  yrange                : two-element array of minimum and maximum y values
;  xunit                 : unit to plot on x-axis
;  yunit                 : unit to plot on y-axis
;  inset                 : set to zero to hide SFH inset, or 1-4 to choose which corner to place it in
;  stripe                : show diagonal stripes indicating lines of constant magnitude or flux
;  orient                : fiddle with the caption orientation for stripes
;  x2unit                : unit to plot on RIGHT x-axis (must linearly scale with xunit)
;  y2unit                : unit to plot on TOP y-axis (must linearly scale with yunit)
;  ntrials               : run a Monte Carlo uncertainty analysis with this many trials
;  fitzrange             : range to consider for fitting the photometric redshift.  Deprecated?  Photo-z capabilities not well tested.
;  alsoplot              : filename to show some other SED on the plot (i.e. for comparison)
;  residualsinfo         : print out a table showing the residuals in each photometric filter
;  minage                : minimum population age to permit fit (to avoid convergence to infinitely young starbursts)
;  minunc                : set any fractional uncertainties less than this fraction to this fraction
;  addunc                ; add this much fractional uncertainty to every point
;  resolution            ; resolution for models - higher numbers run faster but are less accurate when dealing with narrow lines


 ; set some defaults

outplotfile = 'sedfit.eps' ; also copied to plots/
outmodelphotfile = 'modelphot.dat' ; also copied to modelphot/
outsedfile = 'modelsed.txt'

if keyword_set(loud) then quiet = 0 else quiet = 1
if n_elements(resolution) eq 0 then resolution = 11
if n_elements(minage) eq 0 then minage = 1.0e7 
minage2 = 5.0e7

if n_elements(minunc) eq 0 then minunc = 0.01
if n_elements(addunc) eq 0 then addunc = 0.02


; Load photometry
if n_elements(filter) gt 0 and n_elements(flux) eq n_elements(filter) and n_elements(fluxerr) eq n_elements(flux) then begin
   if n_elements(photometryfile) gt 0 then begin 
      print, 'Cannot specify filter/flux/fluxerr and photometryfile simultaneously.'
      return
   endif
   obs = replicate({galaxy: '', filter:'', wav:0.0,  emag:0.0, stdmag:0.0, abmag:0.0, $
                 uplim:0, flux:0.0, fluxerr:0.0, fwhmwav:0.0, tel:'', date:'', ref:'', nofit:0},n_elements(filter))
   obs.filter = filter
   obs.wav = filtwv(filter, fwhm=fwhmwav)
   obs.fwhmwav = fwhmwav
   obs.flux = flux
   obs.fluxerr = fluxerr
   obs.stdmag = ujy2mag(flux,filter)
   obs.abmag = ujy2mag(flux,'AB')
   obs.emag = ujy2mag(flux,filter)-ujy2mag(flux+fluxerr,filter)
   galaxy = ''
endif
if n_elements(photometryfile) gt 0 then begin
   if file_test(photometryfile) eq 0 then begin
     print, ' Cannot find '+photometryfile
     return
   endif
   obs = readgalphot(photometryfile, minunc=minunc, addunc=addunc)  
   obs = obs[where(obs.filter ne '')]
   nobsgalaxies = n_elements(unique(obs.galaxy))
   w = where(obs.galaxy eq galaxy, ct)
   if ct eq 0 then begin
     print, galaxy+' is not in '+photometryfile+' - check spelling.'
   endif
   obs = obs[w]
endif

; Load long-wavelength photometry
if keyword_set(longfile) then begin
   longobs = readlongdata(longfile)
   wlong = where(longobs.galaxy eq galaxy, nlong)
   if nlong gt 0 then begin
      longobs = longobs[wlong]
   endif else begin
      print, 'No long-wavelength data.'
      delvarx, longobs
   endelse
endif

; set BC03 model parameters
setbc03, binning=resolution, resolution=resolution

; load model guesses/constraints
if n_elements(modelfile) gt 0 then begin
   if file_test(modelfile) eq 0 then begin
      print, modelfile+' not found.'
      return
   endif
   modelval = loadgalmodels(modelfile, fixed=modelfix)
   if size(modelval,/tn) ne 'STRUCT' then begin
      print, modelfile+' has a formatting error and cannot be read successfully.'
      return
   endif
   wg = where(modelval.galaxy eq galaxy, ctwg)
   if ctwg eq 0 then begin
     print, galaxy+' is not in '+modelfile+' - check spelling.'
     return
   endif
   modelval = modelval[wg]
   modelfix = modelfix[wg]
   nmodels = n_elements(modelval)
endif else begin
  print, 'No model file specified:  using generic initial guess'
  modelval = loadgalmodels(fixed=modelfix)
  modelval.z = z               &  modelfix.z = 1
  modelval.SFR = 1             &  modelfix.SFR = 0
  modelval.tav = -1.           &  modelfix.tav = 1
  modelval.age = 1e8           &  modelfix.age = 1
  modelval.age2 = galage(z,10,/silent)   &  modelfix.age2 = 1
  modelval.mass = (1e8*modelval.SFR)*2   &  modelfix.mass = 0
  modelval.tau = -1.           &  modelfix.tau = 1
  modelval.tsmooth = 0         &  modelfix.tsmooth = 1
  modelval.dustlaw = 'calz'    &  modelfix.dustlaw = 1
  modelval.Av = 0.0            &  modelfix.Av = 1
  modelval.Avembed = 50.         &  modelfix.Avembed = 1
  modelval.Rv = -1.0             &  modelfix.Rv = 1
  modelval.fembed = 0.0          &  modelfix.fembed = 1
  modelval.Tdust = 40            &  modelfix.Tdust = 1
  modelval.metal = 1.0           &  modelfix.metal = 1
  modelval.gasmetal = 1.0        &  modelfix.gasmetal = 1
  modelval.q = 1e7               &  modelfix.q = 1
  modelval.L = -1.               &  modelfix.L = 1
  nmodels = 1
endelse

ppos = ppos()


if keyword_set(replot) then goto, replot

npar = ppos.npar
allsol = fltarr(nmodels, npar)
allerr = fltarr(nmodels, npar)
allchisq = fltarr(nmodels)
alldof = intarr(nmodels)

seed = 2391
if n_elements(ntrials) gt 0 then uncertaintyanalysis = 1

for m = 0, nmodels-1 do begin
   maxage = galage(modelval[m].z, 20., /silent)

   parinfo = mkparinfo(modelval[m], modelfix[m], minage=[minage, minage2], maxage=maxage)
   init = parinfo.value
   fitobs = obs[where(obs.wav lt 8*10000. and obs.nofit eq 0)]
   ;fitobs = obs[where(obs.wav lt 20*10000.)]

   ; could do a check here to make sure parinfo is consistent with limits.

   sol = mpfitfun('buildgalphot', fitobs.filter, fitobs.flux, fitobs.fluxerr, parinfo=parinfo,$
               perr=perr, bestnorm=chisq, dof=dof, yfit=yfit, quiet=quiet)
   npar = n_elements(sol)    
   recalcchisq = total(((fitobs.flux-yfit)/(fitobs.fluxerr))^2)  ; for some reason the fitter returns a wrong value sometimes
   if n_elements(perr) eq 0 then begin
      print, 'Fitting failed!!'
      return
   end

   if keyword_set(residualsinfo) then begin
      print, ' filt        f_fit        f_obs        f_unc        res/unc        chi^2'
      for i = 0, n_elements(fitobs)-1 do begin
        print, rclip(fitobs[i].filter,5)+'  ', $
            yfit[i], $
            fitobs[i].flux, $
            fitobs[i].fluxerr, $
            (fitobs[i].flux-yfit[i])/fitobs[i].fluxerr, $
            ((yfit[i]-fitobs[i].flux)/fitobs[i].fluxerr)^2
      endfor
      print, clip('',59),  recalcchisq
   endif

   fullsol = fullsol(sol)
   ; full sol is the sol without the -1's for alternate parameterizations.
   ; not really necessary since we are using monte-carlo method with derived pars.

   negperr = perr*0 ; default
   posperr = perr*0

   if m gt 0 then mstr = '.'+clip(m) else mstr = ''
   
   outsolstr = clip(galaxy+mstr,9)
   for s = 0, n_elements(sol)-1 do outsolstr += ' ' + clip(string(sol[s]),12)
   updatefile, 'outsol.dat', outsolstr

   if m gt 0 then mstr = '.'+clip(m) else mstr = ''

   ; Fit parameters
   par = [ppos.z, ppos.SFR, ppos.age, ppos.age2, ppos.tau, ppos.mass, ppos.Av, ppos.dustlaw, ppos.Avembed,  ppos.fembed, ppos.Tdust, ppos.metal, ppos.gasmetal, ppos.q]
   parstr = clip(galaxy+mstr,9)
   for p = 0, n_elements(par)-1 do begin
     parstr += ' ' + clipzero(float(sol[par[p]]))+ clip(repstr(repstr(string(parinfo[par[p]].fixed and sol[par[p]] ge 0),'1','*'),'0',''))
   endfor
   updatefile, 'outmodelpar.dat', parstr



   if keyword_set(uncertaintyanalysis) then begin
      ; This is slightly slower but more robust than the root-finding method before.

      if file_test('trials/') eq 0 then spawn, 'mkdir trials'
      openw, 7, 'trials/'+galaxy+mstr+'.trial.dat'
      outtstr = clip('[sol]',5)
      for s = 0, n_elements(sol)-1 do outtstr += ' ' + clip(string(sol[s]),12)
      printf, 7, outtstr

      alltrialsol = fltarr(ntrials, n_elements(sol)) + !values.f_nan
      dummy =  clip(0)+fpr(0,1.0)+rclip(0)+epr(2.2,2) ; get the compile notices out of the way
      cprint, ' '
      cprint, 'Trial '
      s = ''
      info = ''
      nfail = 0
      nrepeat = 0
      for t = 0, ntrials-1 do begin
        for r = 0, strlen(s)+strlen(info) do cprint, '\b'
        s = rclip(t+1,4)+'/'+clip(ntrials,4)
        cprint, s
        parinfo.value = sol
        ;parinfo[ppos.age].value *= 0.95  ; why was this here?  It causes failure when age is set to the minimum age.
        if n_elements(fitzrange) eq 2 then $
           parinfo[ppos.z].value = fitzrange[0]+(fitzrange[1]-fitzrange[0])*t*1.0/ntrials ; doesn't need to be random
        modflux = fitobs.flux + fitobs.fluxerr * randomn(seed, n_elements(fitobs.flux))
        w = where(modflux lt 0, ct)
        trialsol = mpfitfun('buildgalphot', fitobs.filter, modflux, fitobs.fluxerr, parinfo=parinfo,$
                  perr=trialperr, bestnorm=trialchisq, dof=trialdof, yfit=trialyfit, quiet=quiet)
        if n_elements(trialsol) eq 1 or trialdof lt 0 then begin
           info = ': FAILED'
           cprint, info
           nfail += 1
           if nfail gt 100 then begin
             print, 'Too many failures; stopping'
             break
           endif
           t = t - 1
           continue
        endif
        ismatch = 0
        for tp = 0, t-1 do begin
           ismatch = total(trialsol eq alltrialsol[tp,*]) eq n_elements(trialsol) ; identical to a previous run?
           if ismatch then break
        endfor
        if ismatch then begin
           info = ': repeat solution (matches trial '+clip(tp)+')'+clip('',30)
           cprint, info
           nrepeat += 1
           break
        endif
        truetrialchisq = total( ((fitobs.flux-trialyfit)/fitobs.fluxerr)^2 )
        fulltrialsol = fullsol(trialsol)
        outtstr = clip('['+clip(t)+']',5)
        for s = 0, n_elements(trialsol)-1 do outtstr += ' ' + clip(string(trialsol[s]),12)
        printf, 7, outtstr
        alltrialsol[t,*] = trialsol
        nochange = total(abs(trialsol-sol)) eq 0
        if nochange then changestr = ' - no change' else changestr = ''
        info = '  ... z:' + fpr(trialsol[ppos.z],1.2) + ' SFR:' + clip(fpr(trialsol[ppos.sfr],3.2),6) + ' Mass:' + epr(fulltrialsol[ppos.mass],2) + ' Av:'+fpr(fulltrialsol[ppos.av],1.2) + ' age:'+epr(fulltrialsol[ppos.age] > 0,2) + '  Chi^2:' + fpr(truetrialchisq,3.1) + '/' + clip(trialdof) + changestr
        cprint, info

        ;dum = buildgalphot(fitobs.filter, trialsol, wav=wav, fnu=fnu, lbol=lbol)
        ;redshift, wav, fnu, sol[ppos.z], restwav, fnu_10pc, 0.

      endfor
      print, ' - done'
      close, 7
      if nfail gt 0 then print, nfail, ' failed trials.'
      if nrepeat gt 0 then print, nrepeat, ' non-unique solutions.'
      nonrepeat = where(finite(alltrialsol[*,1]), ct)
      alltrialsol = alltrialsol[nonrepeat,*]

      for p = 0, ppos.npar-1 do begin
        tsol = alltrialsol[*,p]
        if max(tsol)-min(tsol) eq 0 then continue
        posperr[p] = -sol[p] + percentile(tsol, 84.15) ; 1-sigma
        negperr[p] =  sol[p] - percentile(tsol, 15.85)
      endfor

      if file_test('sav/') eq 0 then spawn, 'mkdir sav'
      save, sol, alltrialsol, filename='sav/'+galaxy+mstr+'trials.sav'  ; model fit parameters only

 
      ; File with uncertainties

      parstr = clip(galaxy+mstr,9)
      parstr += ' | '
      parstr += clip(sol[ppos.z],6)
      parstr += ' | '
      p = ppos.sfr
      parstr += fpr(sol[p],3.3)+' '+fpr(posperr[p],3.3)+' '+fpr(negperr[p],3.3)
      parstr += ' | '
      p = ppos.mass
      parstr += fpr(sol[p]/1e9,4.3)+' '+fpr(posperr[p]/1e9,4.3)+' '+fpr(negperr[p]/1e9,4.3)
      parstr += ' | '
      p = ppos.age
      parstr += fpr(sol[p]/1e6,4.2)+' '+fpr(posperr[p]/1e6,4.2)+' '+fpr(negperr[p]/1e6,5.2)
      parstr += ' | '
      p = ppos.av
      parstr += fpr(sol[p],4.2)+' '+fpr(posperr[p],4.2)+' '+fpr(negperr[p],5.2)
      parstr += ' | '
      parstr += fpr(recalcchisq,3.1) + '/' + clip(dof)
      parstr += ''
      updatefile, 'outmodelparunc.dat', parstr

   endif



   if nmodels eq 1 then begin
      print, 'Parameters:'
   endif else begin
      print, 'Model '+clip(m)+':'
   endelse

   print, parstring('Redshift', 'z', sol[ppos.z], fullsol[ppos.z], parinfo[ppos.z].fixed, perr[ppos.z], posperr[ppos.z], negperr[ppos.z])
   if sol[ppos.metal] ge 0 and parinfo[ppos.metal].fixed eq 0  then $
   print, parstring('Stellar metallicity', 'Z*/sol', sol[ppos.metal], fullsol[ppos.metal], parinfo[ppos.metal].fixed, perr[ppos.metal], posperr[ppos.metal], negperr[ppos.metal])
   if sol[ppos.gasmetal] ge 0 and parinfo[ppos.gasmetal].fixed eq 0  then $
   print, parstring('Nebular metallicity', 'Z~/sol', sol[ppos.gasmetal], fullsol[ppos.gasmetal], parinfo[ppos.gasmetal].fixed, perr[ppos.gasmetal], posperr[ppos.gasmetal], negperr[ppos.gasmetal])
   print, parstring('Stellar mass', 'M/sol', sol[ppos.mass], fullsol[ppos.mass], parinfo[ppos.mass].fixed, perr[ppos.mass], posperr[ppos.mass], negperr[ppos.mass])
   print, parstring('Current star-formation rate', 'SFR', sol[ppos.sfr], fullsol[ppos.sfr], parinfo[ppos.sfr].fixed, perr[ppos.sfr], posperr[ppos.sfr], negperr[ppos.sfr])
   if sol[ppos.age2] lt 0 and sol[ppos.tav] lt 0 then begin
     print, parstring('Maximum age', 'age', sol[ppos.age], fullsol[ppos.age], parinfo[ppos.age].fixed, perr[ppos.age], posperr[ppos.age], negperr[ppos.age])
   endif else begin
     if sol[ppos.tav] gt 0 then begin
     print, parstring('Average age', 'age', sol[ppos.tav], fullsol[ppos.tav], parinfo[ppos.tav].fixed, perr[ppos.tav], posperr[ppos.tav], negperr[ppos.tav])
     endif else begin
     print, parstring('Population 1 age', 'age1', sol[ppos.age], fullsol[ppos.age], parinfo[ppos.age].fixed, perr[ppos.age], posperr[ppos.age], negperr[ppos.age])
     print, parstring('Population 2 age', 'age2', sol[ppos.age], fullsol[ppos.age2], parinfo[ppos.age2].fixed, perr[ppos.age2], posperr[ppos.age2], negperr[ppos.age2])
     endelse 
   endelse
   if sol[ppos.tau] ge 0 and modelfix[m].tau eq 0 then $
     print, parstring('Star-formation timescale', 'tau', sol[ppos.tau], fullsol[ppos.tau], parinfo[ppos.tau].fixed, perr[ppos.tau], posperr[ppos.tau], negperr[ppos.tau])
   print, parstring('Extinction', 'Av', sol[ppos.av], fullsol[ppos.av], parinfo[ppos.av].fixed, perr[ppos.av], posperr[ppos.av], negperr[ppos.av])
   print, '                        Chi^2/dof = ', recalcchisq, ' / ', clip(dof)
   

   allsol[m,*] = sol
   allerr[m,*] = perr
   allchisq[m] = recalcchisq
   alldof[m] = dof

   ;initsynphot = buildgalphot(obs.filter, init, wav=wav, flambda=flambda)
   ;addsedplot, wav, flambda, xunit='ang', yunit='flambda/ang', style=1, color=0, thick=1


endfor


if file_test('sav/') eq 0 then spawn, 'mkdir sav'
save, allsol, allerr, allchisq, alldof, parinfo, galaxy, filename='sav/'+clip(galaxy)+'sol.sav'
replot:
oldgalaxy = galaxy
if keyword_set(replot) and file_test('sav/'+clip(galaxy)+'sol.sav') then $
   restore, 'sav/'+clip(galaxy)+'sol.sav'
if galaxy ne oldgalaxy then print, 'Host in stored solution is not the same as new host.'

!p.multi = [0, 1, 2]
!p.position = [0.13,0.13,0.89,0.9]

   ; can now do this at the end...
openplot, filename=outplotfile, xs=5.5, ys=4

if n_elements(xunit) eq 0 then xunit = 'um'
if n_elements(yunit) eq 0 then yunit = 'nufnu'
if xunit eq 'um'   and n_elements(x2unit) eq 0 then x2unit = 'rest_um'
if yunit eq 'nufnu'and n_elements(y2unit) eq 0 then y2unit = 'nuLsolnu'
if n_elements(stripe) eq 0 then stripe = 'mag'
if n_elements(orient) eq 0 then orient = -20

if n_elements(xrange) eq 0 then begin
   if xunit eq 'um' then xrange = [0.2,6]
endif
if n_elements(yrange) eq 0 then begin
   ;if yunit eq 'nufnu' then yrange = [2e-16,1e-13]
   wv = where(obs.wav gt 1000 and obs.wav lt 10d4, ct)
   if ct gt 0 then begin
     if yunit eq 'fnu' then yrange = median(obs[wv].flux)*[22,1./22]
     if yunit eq 'nufnu' then yrange = median((obs[wv].flux > obs[wv].fluxerr*2)*3d-11/obs[wv].wav)*[1./22,22]
   endif
endif

;if n_elements(xrange) eq 0 then convert, [0.3, 9.], empty, xrange, xunit='um'
;if n_elements(yrange) eq 0 then convert, xrange, [2e-16,1e-13], empty2, yrange, xunit=xunit, yunit='nufnu'
if n_elements(inset) eq 0 then inset = 2

sedplot, xrange=xrange, yrange=yrange, xunit=xunit, yunit=yunit, x2unit=x2unit, y2unit=y2unit, z=modelval[0].z, stripe=stripe, orient=orient

;sedplot, xrange=xrange, yrange=yrange, xunit='um', yunit='flambda/ang', x2unit='rest_um', z=modelval[0].z, stripe='mag', orient=-20





 


   print
   for m = 0, nmodels-1 do begin
      sol = allsol[m,*]
      err = allerr[m,*]

      par = [ppos.z, ppos.SFR, ppos.age, ppos.age2, ppos.mass, ppos.tau, ppos.dustlaw, ppos.Av, ppos.Avembed,  ppos.fembed, ppos.Tdust, ppos.metal]
      if sol[ppos.age] le 0 then age = sol[ppos.mass]/sol[ppos.SFR]
      if sol[ppos.age] gt 0 and sol[ppos.age] le 1e8 then age = sol[ppos.age2]
      if sol[ppos.age] gt 1e8 then age = sol[ppos.age]
      if sol[ppos.age] le 0 then ageerr = sol[ppos.mass]/sol[ppos.SFR]*sqrt((err[ppos.SFR]/sol[ppos.SFR])^2 + (err[ppos.mass]/sol[ppos.mass])^2)
      if sol[ppos.age] gt 0 and sol[ppos.age] le 1e8 then ageerr = err[ppos.age2]
      if sol[ppos.age] gt 1e8 then ageerr = err[ppos.age]
      dlaw = (extmap()) [sol[ppos.dustlaw]]
      avsfrerr = sol[ppos.mass]/age*sqrt((ageerr/age)^2 + (err[ppos.mass]/sol[ppos.mass])^2)
      if sol[ppos.tau] gt 0 then tau = clipzero(sol[ppos.tau]/1e6) else tau = '\infty'
      if sol[ppos.fembed] gt 0 then begin
         avstr = '$'+fpr(sol[ppos.Av],2.1)+'\pm'+fpr(err[ppos.Av],1.1)+'$ & $'+fpr(sol[ppos.Avembed],2.1)+'\pm'+fpr(err[ppos.Avembed],1.1)+'$     '
      endif else begin
         avstr = '\multicolumn{2}{c|}{$'+fpr(sol[ppos.Av],1.1)+'\pm'+fpr(err[ppos.Av],1.1)+'$}'
      endelse
      if keyword_set(latex) then begin
      print, '$'+fpr(sol[ppos.gasmetal],1.2)+'\pm'+fpr(err[ppos.gasmetal],1.2)+'$ & '+ $
             '$'+fpr(sol[ppos.SFR],3.1)+'\pm'+fpr(err[ppos.SFR],2.1)+'$ & '+ $
             '$'+fpr(sol[ppos.mass]/age,3.1)+'\pm'+fpr(avsfrerr,2.1)+'$ & '+ $
             '$'+fpr(age/1e9,3.1)+'\pm'+fpr(ageerr/1e9,2.1)+'$ & '+ $
;             fpr(sol[ppos.age2]/1e9,3.1)+'\pm'+ fpr(err[ppos.age2]/1e9,3.1)+'$ & '+ $
             '$'+clip(tau,7)+'$ & '+ $
             '$'+clip(clipzero(round(sol[ppos.mass]/1d9)),3)+'\pm'+clip(clipzero(round(err[ppos.mass]/1d9)),3)+'$ & '+ $
                clip(dlaw,5)+ ' & ' $
              + avstr + ' & ' $
              + fpr(allchisq[m],2.2)+' / '+clip(alldof[m]) +' \\'
      endif
   endfor

   ; Add model curves
   
   modelval.linecolor = ([0,5,9,7]) [0:n_elements(modelval.linecolor)-1]
   if file_test('modelphot/') eq 0 then spawn, 'mkdir modelphot'

   nh = 0
   ;for m = nmodels-1, 0, -1 do begin
    for m = 0, nmodels-1 do begin
      sol = allsol[m,*]

      ;print, sol

      solsynphot = buildgalphot(obs.filter, sol, wav=wav, flambda=flambda, hist=hist, lbol=lbol)
      if m gt 0 then mstr = clip(m) else mstr = ''
      save, filename='sav/'+galaxy+'sed'+mstr+'.sav', wav, flambda
      if galaxy ne '' then begin
         openw, 3, outsedfile
         printf, 3, '#'+galaxy
         printf, 3, '#wav(Ang)     flambda(cgs)'
         for i = 0, n_elements(wav)-1 do begin
            printf, 3, wav[i], flambda[i]
         endfor
         close, 3
         if galaxy ne '' then spawn, 'cp -f '+outsedfile+' modelphot/'+galaxy+'modelsed.txt'
      endif
      addsedplot, wav, flambda, xunit='ang', yunit='flambda/ang', color=modelval[m].linecolor, thick=modelval[m].linethick, style=modelval[m].linestyle
      size = 0.8-0.25*(obs.fwhmwav lt 200.)
      addsedmegaplot, obs.wav, solsynphot, xunit='ang', yunit='uJy', psym=8, size=size, color=modelval[m].linecolor;+1

      nf = n_elements(obs.wav)
      isline = intarr(nf)
      for f = 0, nf-1 do begin
        if strlen(obs[f].filter) lt 2 then continue
        if strmid(obs[f].filter,0,2) eq 'H_' or strmid(obs[f].filter,0,1) eq '[' then begin
          isline[f] = 1
        endif
      endfor

      if galaxy ne '' then begin
        openw, 3, outmodelphotfile
        printf, 3, 'filter'+clip('wav',11)+clip('flux',11)+clip('synflux',11)+clip('err',11)+clip('diff',11)+clip('sigma',11)
        for f = 0, n_elements(obs.wav)-1 do begin
          if isline[f] eq 0 then $
          printf, 3, clip(obs[f].filter,6)+clip(obs[f].wav,11)+clip(obs[f].flux,11)+clip(solsynphot[f],11)+clip(obs[f].fluxerr,11)+clip(solsynphot[f]-obs[f].flux,11)+clip((solsynphot[f]-obs[f].flux)/obs[f].fluxerr,11)

          if isline[f] eq 1 then $
          printf, 3, clip(obs[f].filter,6)+clip('',11)+clip(obs[f].flux/1d-17,11)+clip(solsynphot[f]/1d-17,11)+clip(obs[f].fluxerr/1d-17,11)+clip((solsynphot[f]-obs[f].flux)/1d-17,11)+clip((solsynphot[f]-obs[f].flux)/obs[f].fluxerr,11)

        endfor
        close, 3
        if galaxy ne '' then spawn, 'cp -f '+outmodelphotfile+' modelphot/'+galaxy+'modelphot.txt'
      endif

      ;print, float(    wav[where(wav gt 6545*(1+z) and wav lt 6580*(1+z))])
      ;print, float(flambda[where(wav gt 6545*(1+z) and wav lt 6580*(1+z))])

      oldyoung = 0
      if oldyoung then begin
       lage = (shift(hist.t, 1) + hist.t)/2.
       uage = (shift(hist.t,-1) + hist.t)/2.
       lage[0] = 0.
       uage[n_elements(hist.t)-1] = max(hist.t)*1.2
       dage = (uage - lage)
 
       wyoung = where(hist.t lt 100e6, complement=wold)
       younghist = hist
       younghist.sfr[wold] = 0
       younghist.sfrobs[wold] = 0
       youngmass = sol[ppos.mass] * total((younghist.sfr)*dage) / total((hist.sfr)*dage)
       oldhist = hist
       oldhist.sfr[wyoung] = 0
       oldhist.sfrobs[wyoung] = 0
       oldmass = sol[ppos.mass] * total((oldhist.sfr)*dage) / total((hist.sfr)*dage)
 
       buildgalspec, youngfreq, youngfnu, z=sol[ppos.z], mass=youngmass, metallicity=sol[ppos.metal], $
                    embedfrac=sol[ppos.fembed], av=sol[ppos.av], embedav=sol[ppos.avembed], dustlaw=modelval[m].dustlaw, $
                    Tdust=sol[ppos.Tdust], binning=br, resolution=br, sethist=younghist
       addsedplot, youngfreq, youngfnu, xunit='GHz', yunit='uJy', color=modelval[m].linecolor, style=1
 
       buildgalspec, oldfreq, oldfnu, z=sol[ppos.z], mass=oldmass, metallicity=sol[ppos.metal], $
                    embedfrac=sol[ppos.fembed], av=sol[ppos.av], embedav=sol[ppos.avembed], dustlaw=modelval[m].dustlaw, $
                    Tdust=sol[ppos.Tdust], binning=br, resolution=br, sethist=oldhist
       addsedplot, oldfreq, oldfnu, xunit='GHz', yunit='uJy', color=modelval[m].linecolor, style=2
      endif

      if nh eq 0 then begin
       nh = n_elements(hist.t) ; 221
       sft       = hist.t
       sfhist    = fltarr(nmodels,nh) ; store the star-formation history
     endif
     sfhist[m,*] = hist.sfr
   endfor


   if n_elements(alsoplot) gt 0 then begin
      readcol, alsoplot, alsowav, alsoflux, format='f,f'  ; confusingly is rest-frame wav and observed flux
      addsedplot, alsowav*(1+modelval[0].z), alsoflux, xunit='um', yunit='Jy', color=0, thick=1, style=1
   endif


  
   ; Add observations

   fillcolor = 14
   nofitcolor = 18

   flux =  obs.flux       ;mag2ujy(obs.abmag,'AB')
   uncflux = obs.fluxerr  ;mag2ujy(obs.abmag-obs.emag,'AB')-flux
   yperr = -obs.abmag+ujy2mag(flux-uncflux,'AB')
   size = 0.9 ;-0.3*(obs.fwhmwav lt 200.)
   addsedmegaplot, obs.wav, obs.abmag, xunit='ang', yunit='AB', xerr=obs.fwhmwav/2., yperr=yperr, ymerr=obs.emag, limit=obs.uplim, size=size
   wf = where(obs.nofit eq 0, ctf, complement=wnf)
   if ctf gt 0 then addsedmegaplot, obs[wf].wav, obs[wf].abmag, xunit='ang', yunit='AB', fill=1, color=fillcolor, size=size
   if ctf lt n_elements(obs) then $
                    addsedmegaplot, obs[wnf].wav, obs[wnf].abmag, xunit='ang', yunit='AB', fill=1, color=nofitcolor, size=size
   addsedmegaplot, obs.wav, obs.abmag, xunit='ang', yunit='AB', limit=obs.uplim, size=size

   for j = 0, n_elements(longobs)-1 do begin
      if longobs[j].limstr eq '=' then begin
         addsedmegaplot, longobs[j].freq, longobs[j].flux, xunit='GHz', yunit='uJy', yerr=longobs[j].fluxerr*2 ; note 2-sigma error bar
         addsedmegaplot, longobs[j].freq, longobs[j].flux, xunit='GHz', yunit='uJy', color=fillcolor, /fill ;
         addsedmegaplot, longobs[j].freq, longobs[j].flux, xunit='GHz', yunit='uJy' ;
      endif
      if longobs[j].limstr eq '~' then begin
        ;addsedmegaplot, longobs[j].freq, longobs[j].flux, xunit='GHz', yunit='uJy', yperr=longobs[j].fluxerr*2., ymerr=longobs[j].fluxerr*0.001 ; again 2-sigma
        ;addsedmegaplot, longobs[j].freq, longobs[j].flux, xunit='GHz', yunit='uJy', /fill, color=fillcolor
        ;addsedmegaplot, longobs[j].freq, longobs[j].flux, xunit='GHz', yunit='uJy', /limit
        addsedmegaplot, longobs[j].freq, longobs[j].flux+longobs[j].fluxerr*2, xunit='GHz', yunit='uJy', color=fillcolor, /fill
        addsedmegaplot, longobs[j].freq, longobs[j].flux+longobs[j].fluxerr*2, xunit='GHz', yunit='uJy', yerr=longobs[j].fluxerr*longobs[j].freq, /limit
      endif
      if longobs[j].limstr eq '<' then begin
         addsedmegaplot, longobs[j].freq, longobs[j].flux, xunit='GHz', yunit='uJy', color=fillcolor, /fill
         addsedmegaplot, longobs[j].freq, longobs[j].flux, xunit='GHz', yunit='uJy', /limit
      endif
   endfor

   ; Add SF history inset

  if inset gt 0 then begin
   ;histpos = [0.1, 0.65, 0.35, 0.85]
   ;histpos = [0.34, 0.21, 0.58, 0.42]
   if inset eq 1 then histpos = [0.50, 0.21, 0.85, 0.47] ; bottom right
   if inset eq 2 then histpos = [0.50, 0.62, 0.85, 0.88] ; top right
   if inset eq 3 then histpos = [0.20, 0.21, 0.55, 0.47] ; bottom left
   if inset eq 4 then histpos = [0.20, 0.62, 0.55, 0.88] ; top left
   !p.position = histpos
   polyfill, histpos[[0,2,2,0,0]], histpos[[1,1,3,3,1]], /norm, color=18
   minsf = min(sfhist[where(sfhist gt 0)])
   maxsf = max(sfhist)
   maxt = 1.1*max(sft[where(sfhist gt 0)]) > 6e9
   plot, [0], [0], xrange=[6e6, maxt], yrange=[minsf*0.95 < 1.0 < maxsf*0.05, maxsf*(maxsf/minsf)^0.05], /xstyle, charsize=0.6, /xlog, /ylog, $
     xtitle = 'Time before observed (yr)', ytitle='Star-formation rate SFR(t)', /ystyle
   for m = nmodels-1, 0, -1 do begin

     oplot, sft, sfhist[m,*] > yrange[0]*0.9, linestyle=modelval[m].linestyle, color=modelval[m].linecolor, thick=modelval[m].linethick
   endfor
  endif


   device, /close
   set_plot, 'X'
   !p.multi = 0
   
   retitleplot, outplotfile, galaxy+' host SED fit'

   if file_test('plots/') eq 0 then spawn, 'mkdir plots'
   outplotname = 'plots/'+galaxy+'hostfit.eps' 
   if file_test(outplotname) then spawn, 'rm -f '+outplotname


   if galaxy ne '' then spawn, 'cp '+outplotfile+' '+outplotname


end

