function loadbc03, binning=binning, resolution=resolution

  common rawbc03block, bc03
  common galbuilder, galbuilderpath
  if n_elements(galbuilderpath) eq 0 then galbuilderpath = getgalbuilderpath()

  ;help, binning
  ;help, resolution
  
  if n_elements(bc03) eq 0 then begin
     print, 'loading...'
     ;restore, '~/research/popsyn/bc03/models/Padova1994/chabrier/bc03hr.sav'
     restore, galbuilderpath+'bc03hr.sav'  ; this is not generally where the loading happens - see popsynhist
  endif

  if n_elements(binning) eq 0 and n_elements(resolution) eq 0 then return, bc03



  if n_elements(resolution) gt 0 and n_elements(binning) eq 0 then binning = 1.0

  print, 'binning...'

  ; Rebin to the desired bin width

  nf = 6
  nage = 221
  nwav = 6900

  ; below 3325
  minwav = 91.
  maxwav = 9970.  ; I think above here the native bc03 width is larger than 1 ang anyway...?


  ; Temporarily interpolate to a smaller, uniform bin width to enable binning to 
  ;  the requested uniform width

  if binning eq floor(binning) and binning gt 1.0 then begin
    tempbin = 1.0  ; temporarily bin to 1 Angstrom 
  endif else begin
    tempbin = binning / ceil(binning) ; temporarly bin to something less than 1 ang that is an integer multiple
  endelse
  fbin = binning/tempbin ; binning factor, should always be an integer

  ;usemaxwav = minwav + tempbin * floor((maxwav-minwav)/tempbin) ; make sure it's an integer

  ntempwav = floor((maxwav-minwav)/tempbin) + 1
  if ntempwav mod fbin ne 0 then ntempwav += (fbin - (ntempwav mod fbin)) ; must be a multiple of fbin
  templowwav = tempbin * findgen(ntempwav)
  nlowwav = n_elements(templowwav)/fbin
  lowwav = rebin(templowwav, nlowwav)
  maxlowwav = max(lowwav)

  newwav = [lowwav, bc03.wav[where(bc03.wav gt maxlowwav)]]
  newnwav = n_elements(newwav)

  bc03new = {metal:fltarr(nf), age:fltarr(nage), wav:fltarr(newnwav), llambda:fltarr(nf,nage,newnwav)}
  bc03new.metal = bc03.metal
  bc03new.age = bc03.age
  bc03new.wav = newwav

  if n_elements(resolution) gt 0 then if resolution gt 0 then begin
     nk = round(((5*resolution)/tempbin)/2)*2+1 ; ensures an odd number, 2.5sigma in each direction
     kernelx = findgen(nk)-nk/2
     kernel = gaussian(kernelx, [1.0,0.,resolution/tempbin])
     kernel /= total(kernel)
  endif

  for f = 0, 6-1 do begin
   for a = 0, 221-1 do begin
      templowflux = interpol(bc03.llambda[f,a,*],bc03.wav,templowwav)

      if n_elements(resolution) gt 0 then templowflux = convol(templowflux,kernel,/edge_trunc)

      lowflux = rebin(templowflux, nlowwav)

      newflux = [lowflux, (bc03.llambda[f,a,where(bc03.wav gt maxlowwav)]) [*]]
 
      bc03new.llambda[f,a,*] = newflux
   endfor
  endfor

  return, bc03new

end

pro testloadbc03
   bc03raw = loadbc03()

   bc03bin = loadbc03(binning=3.0)

   bc03smo = loadbc03(binning=3.0, resolution=6.0)

   !p.multi = 0
   !p.position = 0
   window, 0
  
   plot, bc03raw.wav, bc03raw.llambda[2,50,*], /xlog, /ylog
   oplot, bc03bin.wav, bc03bin.llambda[2,50,*], psym=10
   oplot, bc03smo.wav, bc03smo.llambda[2,50,*], linestyle=2

   window, 1

   plot, bc03raw.wav, bc03raw.llambda[2,50,*], /xlog, /ylog, yrange=[1e-4,1], xrange=[1e2,1e6]
   oplot, bc03bin.wav, bc03bin.llambda[2,50,*], psym=10
   oplot, bc03smo.wav, bc03smo.llambda[2,50,*], linestyle=2 

   window, 2

   plot, bc03raw.wav, bc03raw.llambda[2,50,*], xrange=[3250,4000], /xsty, yran=[0.03,0.06]
   oplot, bc03bin.wav, bc03bin.llambda[2,50,*], psym=10
   oplot, bc03smo.wav, bc03smo.llambda[2,50,*], linestyle=2 

   stop
end


