function loadmodels, filename, fixed=fixed

 ; if filename not specified, returns a blank structure

if file_test(filename) eq 0 then 
   n = 1  $
else $
   n = countlines(filename) $
values = replicate({grb:'', zstr:'', ztype:'', z:-1., SFR:-1., tav:-1., age:-1., age2:-1., mass:-1., tau:-1., tsmooth:-1., dustlaw:'', Av:-1., Avembed:-1., Rv:-1, fembed:-1., Tdust:-1., metal:-1., gasmetal:-1., q:-1., L:-1., linestyle:-1, linecolor:-1, linethick:0.0},n)
cpos = {grb:-1, z:-1, sfr:-1,  tav:-1, age:-1, age2:-1, mass:-1, tau:-1, tsmooth:-1, dustlaw:-1, Av:-1, Avembed:-1, Rv:-1, fembed:-1, Tdust:-1, metal:-1, gasmetal:-1, q:-1, L:-1, linestyle:-1, linecolor:-1, linethick:-1}

clab = {grb:'grb', z:'z', sfr:'sfr', tav:'tav', age:'age', age2:'age2', mass:'mass', tau:'tau', tsmooth:'tsmooth', dustlaw:'dust', Av:'av',  Avembed:'avembed', Rv:'rv', fembed:'fembed', Tdust:'tdust', metal:'metal', gasmetal:'gasmetal', q:'q', L:'lum', linestyle:'style', linecolor:'color', linethick:'thick'}
fixed = replicate({grb:'', zstr:'', ztype:'', z:0, SFR:0, tav:0., age:0, age2:0, mass:0, tau:0, tsmooth:0., dustlaw:1, Av:0, Avembed:0, Rv:0, fembed:0, Tdust:0, metal:0, gasmetal:0, q:0, L:0},n)

dustlaws = extmap()

; Establish internal default behavior :
; user can override with a row with GRB column set to 'default'
; Properties that remain -1 are pushed forward into the model code,
; where they are constrained internally by other parameters
; or indicate special values (e.g. tau of -1 means infinity).
; This only works with SFR-related parameters.
defaultvalue = values[0]
defaultfixed = fixed[0]
defaultvalue.z = 0.                &  defaultfixed.z = 1
defaultvalue.SFR = -1.             &  defaultfixed.SFR = 0
defaultvalue.tav = -1.             &  defaultfixed.tav = 0
defaultvalue.age = -1.             &  defaultfixed.age = 0
defaultvalue.age2 = -1.            &  defaultfixed.age2 = 0
defaultvalue.mass = -1.            &  defaultfixed.mass = 0
defaultvalue.tau = -1.             &  defaultfixed.tau = 0
defaultvalue.tsmooth = 0           &  defaultfixed.tsmooth = 1
defaultvalue.dustlaw = 'calz'      &  defaultfixed.dustlaw = 1
defaultvalue.Av = 0.0              &  defaultfixed.Av = 0
defaultvalue.Avembed = 0.0         &  defaultfixed.Avembed = 0
defaultvalue.Rv = -1.0             &  defaultfixed.Rv = 0
defaultvalue.fembed = 0.0          &  defaultfixed.fembed = 0
defaultvalue.Tdust = 40            &  defaultfixed.Tdust = 0
defaultvalue.metal = 1.0           &  defaultfixed.metal = 0
defaultvalue.gasmetal = 1.0        &  defaultfixed.gasmetal = 0
defaultvalue.q = 1e7               &  defaultfixed.q = 0
defaultvalue.L = -1.               &  defaultfixed.L = 0
defaultvalue.linestyle = 0   
defaultvalue.linecolor = 0
defaultvalue.linethick = 2.5     


if file_test(filename) eq 0 then begin
  fixed = defaultfixed
  return, defaultvalue
endif

openr, 1, filename
i = 0
inline = ''
readf, 1, inline  
inline = clip(inline)
if strlen(inline) eq 0 then begin
   print, 'No header?'
   close, 1
   return, ''
endif
columntitles = strsplit(inline,/extract)
ncol = n_elements(columntitles)
for c = 0, n_elements(columntitles)-1 do begin
   ctitle = strlowcase(columntitles[c])
   cisfix = -1
   if strpos(ctitle,'*') ge 0 then cisfix = 1
   if strpos(ctitle,'~') ge 0 then cisfix = 0
   if strpos(ctitle,'?') ge 0 then cisfix = 0
   ctitle = repstr(ctitle,'*','')
   ctitle = repstr(ctitle,'~','')
   ctitle = repstr(ctitle,'?','')
   if strmid(ctitle,0,1) eq '#' then ctitle = strmid(ctitle,1)
   oldcpos = cpos
   if ctitle eq '|' then continue
   if ctitle eq clab.grb       then cpos.grb = c
   if ctitle eq clab.z         then cpos.z = c
   if ctitle eq clab.sfr       then cpos.sfr = c
   if ctitle eq clab.tav       then cpos.tav = c
   if ctitle eq clab.age       then cpos.age = c
   if ctitle eq clab.age2      then cpos.age2 = c
   if ctitle eq clab.mass      then cpos.mass = c
   if ctitle eq clab.tau       then cpos.tau = c
   if ctitle eq clab.tsmooth   then cpos.tsmooth = c
   if ctitle eq clab.dustlaw   then cpos.dustlaw = c
   if ctitle eq clab.Av        then cpos.Av = c
   if ctitle eq clab.Avembed   then cpos.Avembed = c
   if ctitle eq clab.Rv        then cpos.Rv = c
   if ctitle eq clab.fembed    then cpos.fembed = c
   if ctitle eq clab.Tdust     then cpos.Tdust = c
   if ctitle eq clab.metal     then cpos.metal = c
   if ctitle eq clab.gasmetal  then cpos.gasmetal = c
   if ctitle eq clab.q         then cpos.q = c
   if ctitle eq clab.L         then cpos.L = c
   if ctitle eq clab.linestyle then cpos.linestyle = c
   if ctitle eq clab.linecolor then cpos.linecolor = c
   if ctitle eq clab.linethick then cpos.linethick = c

   if cisfix ge 0 then begin
     if ctitle eq clab.z        then defaultfixed.z = cisfix
     if ctitle eq clab.sfr      then defaultfixed.sfr = cisfix
     if ctitle eq clab.tav      then defaultfixed.tav = cisfix
     if ctitle eq clab.age      then defaultfixed.age = cisfix
     if ctitle eq clab.age2     then defaultfixed.age2 = cisfix
     if ctitle eq clab.mass     then defaultfixed.mass = cisfix
     if ctitle eq clab.tau      then defaultfixed.tau = cisfix
     if ctitle eq clab.tsmooth  then defaultfixed.tsmooth = cisfix
     if ctitle eq clab.dustlaw  then defaultfixed.dustlaw = cisfix
     if ctitle eq clab.Av       then defaultfixed.Av = cisfix
     if ctitle eq clab.Avembed  then defaultfixed.Avembed = cisfix
     if ctitle eq clab.Rv       then defaultfixed.Rv = cisfix
     if ctitle eq clab.fembed   then defaultfixed.fembed = cisfix
     if ctitle eq clab.Tdust    then defaultfixed.Tdust = cisfix
     if ctitle eq clab.metal    then defaultfixed.metal = cisfix
     if ctitle eq clab.gasmetal then defaultfixed.gasmetal = cisfix
     if ctitle eq clab.q        then defaultfixed.q = cisfix
     if ctitle eq clab.L        then defaultfixed.L = cisfix
   endif

   ;if cpos eq oldcpos then print, 'Column not recognized: ', columntitles[c] sadly, does not work because IDL is primitive...
endfor

if cpos.Av lt 0 then print, 'Could not find a column for Av...'
if cpos.Av ge 0 and cpos.dustlaw lt 0 then print, 'Coult not find a column for dust...'
if cpos.z lt 0 then print, 'Could not find a column for redshift...'
for l = 1, n-1 do begin
   readf, 1, inline
   inline = clip(inline)
   if strlen(inline) eq 0 then continue
   if strmid(inline,0,1) eq '#' then continue
   inline = (strsplit(inline,'#',/extract)) [0]
   indata = strsplit(inline,/extract)
   if n_elements(indata) lt ncol then begin
      print, 'Not enough columns; cannot interpret line:'
      print, inline
      continue
   endif
   isfix = intarr(n_elements(indata))-1  ; -1 is default, follow whatever the column header says
   for v = 0, n_elements(indata)-1 do begin
     ;print, v, isfix[v], '  ', indata[v], '  ', strpos(indata[v],'*') ge 0, '  ', repstr(indata[v],'*','')
     if strpos(indata[v],'*') ge 0 then isfix[v] = 1
     if strpos(indata[v],'~') ge 0 then isfix[v] = 0
     if strpos(indata[v],'?') ge 0 then isfix[v] = 0
     indata[v] = repstr(indata[v],'*','')
     indata[v] = repstr(indata[v],'~','')
     indata[v] = repstr(indata[v],'?','')
   endfor

   values[i] = defaultvalue
   fixed[i] = defaultfixed

   values[i].grb = indata[cpos.grb]
   values[i].zstr = indata[cpos.z]
   if strpos(values[i].zstr,'~') ge 0 then values[i].ztype = '~'
   if strpos(values[i].zstr,'?') ge 0 then values[i].ztype = '?'
                               values[i].z         = strnum(indata[cpos.z])
   if cpos.sfr       ge 0 then values[i].sfr       = float(indata[cpos.sfr])
   if cpos.tav       ge 0 then values[i].tav       = float(indata[cpos.tav])
   if cpos.age       ge 0 then values[i].age       = float(indata[cpos.age])
   if cpos.age2      ge 0 then values[i].age2      = float(indata[cpos.age2])
   if cpos.metal     ge 0 then values[i].metal     = float(indata[cpos.metal])
   if cpos.av        ge 0 then values[i].av        = float(indata[cpos.av])
   if cpos.avembed   ge 0 then values[i].avembed   = float(indata[cpos.avembed])
   if cpos.rv        ge 0 then values[i].rv        = float(indata[cpos.ar])
   if cpos.mass      ge 0 then values[i].mass      = float(indata[cpos.mass])
   if cpos.tau       ge 0 then values[i].tau       = float(indata[cpos.tau])
   if cpos.tsmooth   ge 0 then values[i].tsmooth   = float(indata[cpos.tsmooth])
   if cpos.dustlaw   ge 0 then values[i].dustlaw   =       indata[cpos.dustlaw]
   if cpos.fembed    ge 0 then values[i].fembed    = float(indata[cpos.fembed])
   if cpos.Tdust     ge 0 then values[i].Tdust     = float(indata[cpos.Tdust])
   if cpos.gasmetal  ge 0 then values[i].gasmetal  = float(indata[cpos.gasmetal])
   if cpos.q         ge 0 then values[i].q         = float(indata[cpos.q])
   if cpos.L         ge 0 then values[i].L         = float(indata[cpos.L])
   if cpos.linestyle ge 0 then values[i].linestyle = fix(indata[cpos.linestyle])
   if cpos.linecolor ge 0 then values[i].linecolor = fix(indata[cpos.linecolor])
   if cpos.linethick ge 0 then values[i].linethick = float(indata[cpos.linethick])
   
   if cpos.z         ge 0 then if isfix[cpos.z] ge 0 then       fixed[i].z         = isfix[cpos.z]
   if cpos.sfr       ge 0 then if isfix[cpos.sfr] ge 0 then     fixed[i].sfr       = isfix[cpos.sfr]
   if cpos.tav       ge 0 then if isfix[cpos.tav] eq 0 then     fixed[i].tav       = isfix[cpos.tav]
   if cpos.age       ge 0 then if isfix[cpos.age] ge 0 then     fixed[i].age       = isfix[cpos.age]
   if cpos.age2      ge 0 then if isfix[cpos.age2] ge 0 then    fixed[i].age2      = isfix[cpos.age2]
   if cpos.metal     ge 0 then if isfix[cpos.metal] ge 0 then   fixed[i].metal     = isfix[cpos.metal]
   if cpos.av        ge 0 then if isfix[cpos.av] ge 0 then      fixed[i].av        = isfix[cpos.av]
   if cpos.avembed   ge 0 then if isfix[cpos.avembed] ge 0 then fixed[i].avembed   = isfix[cpos.avembed]
   if cpos.rv        ge 0 then if isfix[cpos.rv] ge 0 then      fixed[i].rv        = isfix[cpos.rv]
   if cpos.mass      ge 0 then if isfix[cpos.mass] ge 0 then    fixed[i].mass      = isfix[cpos.mass]
   if cpos.tau       ge 0 then if isfix[cpos.tau] ge 0 then     fixed[i].tau       = isfix[cpos.tau]
   if cpos.tsmooth   ge 0 then if isfix[cpos.tsmooth] ge 0 then fixed[i].tsmooth   = isfix[cpos.tsmooth]
   if cpos.fembed    ge 0 then if isfix[cpos.fembed] ge 0 then  fixed[i].fembed    = isfix[cpos.fembed]
   if cpos.Tdust     ge 0 then if isfix[cpos.Tdust] ge 0 then   fixed[i].Tdust     = isfix[cpos.Tdust]
   if cpos.gasmetal  ge 0 then if isfix[cpos.gasmetal] ge 0 then fixed[i].gasmetal = isfix[cpos.gasmetal]
   if cpos.q         ge 0 then if isfix[cpos.q] ge 0 then       fixed[i].q         = isfix[cpos.q]
   if cpos.L         ge 0 then if isfix[cpos.L] ge 0 then       fixed[i].L         = isfix[cpos.L]

   if strlen(values[i].dustlaw) ge 1 and strlen(values[i].dustlaw) le 2 then begin
      if strmid(values[i].dustlaw,0,1) ge '0' and strmid(values[i].dustlaw,0,1) le '9' then begin
         values[i].dustlaw = dustlaws[fix(values[i].dustlaw)]
      endif
   endif

   if values[i].grb eq 'default' then begin
      defaultvalue = values[i]
      defaultfixed = fixed[i]
   endif else begin
      i += 1
   endelse

endfor

close, 1
if i eq 0 then print, 'No valid models found!!'
values = values[0:i-1]
fixed = fixed[0:i-1]

fixed.grb = values.grb

return, values

end

