function popsynhist, tsfhist, sfhist, model=model, age=age, minage=minage, texp=texp, tsmooth=tsmooth, bct=bct

  ; Input simple temporal parameters (or an arbitrary star-formation history arbitrarily gridded)
  ; returns the SFR at the various BC03 epochs.

  common rawbc03block, bc03
  common galbuilder, galbuilderpath
  if n_elements(galbuilderpath) eq 0 then galbuilderpath = getgalbuilderpath()

  if n_elements(bc03) eq 0 then begin   ; only needed to get the age
     print, 'loading...'
     restore, galbuilderpath+'bc03hr.sav'
  endif

  nage = n_elements(bc03.age)
  sfhist_a = fltarr(nage)

  ; custom star formation history uses t and sfhist
  if n_elements(tsfhist) gt 0 and n_elements(sfhist) gt 0 then begin
     finestep = 100000.
     fineage = finestep/2. + finestep*findgen(1.5e+10/finestep)

  

  ;   sfhist_a = interpol([sfhist,0,0], [tsfhist,max(tsfhist)+1,max(bc03.age)], bc03.age)


    ; not guaranteed to work properly.

    sfhist_fine = interpol(sfhist, tsfhist, fineage)
    print, total(sfhist_fine)
    for f = 0L, n_elements(fineage)-1 do begin   ; why did I do this?
       agediff = bc03.age - fineage[f]
       nearesta = where(abs(agediff) eq min(abs(agediff)))
       sfhist_a[nearesta] += finestep * sfhist_fine[f]
    endfor

     ; normalize to 1 Msun/yr here too??
    lage = (shift(bc03.age, 1) + bc03.age)/2. ; lower limit of bin
    uage = (shift(bc03.age,-1) + bc03.age)/2. ; upper limit of bin
    lage[0] = 0.
    uage[nage-1] = max(bc03.age)*1.2
    dage = (uage - lage)                      ; bin width
    sfhist_a /= total(sfhist_a * dage)

  endif else begin

    ;if n_elements(sfr) gt 0 and n_elements(mass) gt 0 and n_elements(age) eq 0 then age = mass / sfr

    ; various analytic methods

    ; get bin mins, maxes, widths
    lage = (shift(bc03.age, 1) + bc03.age)/2. ; lower limit of bin
    uage = (shift(bc03.age,-1) + bc03.age)/2. ; upper limit of bin
    lage[0] = 0.
    uage[nage-1] = max(bc03.age)*1.2
    dage = (uage - lage)                      ; bin width
  
    agediff = bc03.age - age
    nearesta = (where(abs(agediff) eq min(abs(agediff)))) [0]
    if agediff[nearesta] le 0 then begin
       nearestaminus = nearesta
       nearestaplus = nearesta+1
    endif
    if agediff[nearesta] gt 0 then begin
       nearestaminus = nearesta-1
       nearestaplus = nearesta
    endif
  
    if model eq 'ssp' then begin
       if agediff[nearesta] eq 0 then begin
          sfhist_a[nearesta] = 1.0
       endif else begin
          sfhist_a[nearestaplus]  = abs(agediff[nearestaminus]) /  (abs(agediff[nearestaminus]) + abs(agediff[nearestaplus]))
          sfhist_a[nearestaminus] = 1 -  sfhist_a[nearestaplus]
       endelse
    endif else begin
        ; maximum age cutoff of population
        wyounger = where(bc03.age lt age, cty)
        if cty gt 0 then sfhist_a[wyounger] = 1.0
        sfhist_a[nearesta] = (age - lage[nearesta]) /  dage[nearesta]
        
        ; minumum age cutoff of population
        if n_elements(minage) gt 0 then begin
           agediff = bc03.age - minage
           tooyoung = where(bc03.age lt minage, ct)
           if ct gt 0 then sfhist_a[tooyoung] = 0.0
           minnearesta = (where(abs(agediff) eq min(abs(agediff)))) [0]
           if minnearesta lt nearesta then begin
              sfhist_a[minnearesta] = (uage[minnearesta] - minage) /  dage[minnearesta]
           endif else begin ; treat as an SSP - though there is some inconsistency here as it does not converge to this limit
              meanage = (minage + age)/2.
              agediff = bc03.age - meanage
              if agediff[nearesta] eq 0. then begin
                  sfhist_a[nearesta] = 1.0
              endif else begin
                  sfhist_a[nearestaplus]  = abs(agediff[nearestaminus]) /  (abs(agediff[nearestaminus]) + abs(agediff[nearestaplus]))
                  sfhist_a[nearestaminus] = 1 -  sfhist_a[nearestaplus]
              endelse
           endelse
        endif
  
        if model eq 'cont' or model eq 'cst' then sfhist_a *= 1.0
        if model eq 'exp' then begin
          for a = 0, nage-1 do begin
            sfhist_a[a] *= exp(((bc03.age[a]-age)/texp) < 10.)  ; should really integrate - some sort of weighted value
                   ; the <10 is to prevent infinities.  it should never need to exceed 1.
          endfor
        endif
  
    endelse

    if n_elements(tsmooth) gt 0 then begin
    if tsmooth gt 0 then begin
       ; do an analytic convolution, Gaussian kernel
       smoothedsfhist = fltarr(nage)
       ;;;plot, bc03.age, smoothedsfhist, xrange=[0,1e7], yrange=[0,1]
       for a = 0, nage-1 do begin
          weight = gaussian(bc03.age, [1., bc03.age[a], tsmooth])
          weight = weight / total(weight) ; normalize to 1
          ;if a mod 10 eq 0 then print, a, bc03.age[a]
          ;if a mod 10 eq 0 then oplot, bc03.age, weight
          smoothedsfhist[a] = total(sfhist_a*weight)
       endfor
       sfhist_a = smoothedsfhist
    endif
    endif

   
    sfhist_a /= total(sfhist_a * dage) ; normalize to 1 Msun/yr
  endelse

  bct = bc03.age ; just a convenience
  return, sfhist_a
end

