@popsynhist

; This is the key program to actually create the galaxy SED.

function popsynsed, wavAng, sfhist, sfr=sfr, mass=mass, metallicity=metallicity, Lyoung=Lyoung, Lold=Lold, cutage=cutage, bct=bct, binning=binning, resolution=resolution
  ; sfhist is a special array that is output by popsynhist.
  ; (bct is an output containing the time values of that array)

  common bc03block, bc03, oldbinning, oldresolution
  common bc03default, defaultbinning, defaultresolution

  ; create the SED of an unobscured stellar population with the specified star-formation history.
  ; (more complicated histories can be obtained by summation of the outputs.)

  ; model is cont, ssp, exp, or custom.
  ; cont :            continuous star formation : tau = Inf (or unspecified)
  ;  exp : exponentially falling star formation : tau > 0
  ;  ssp : instantaneous star-formation burst   : tau = 0
  ; custom : any arbitrary history - use popsynhist to make your own sfhist.

  ; tsfhist is in yr
  ; sfhist is in Msol/yr
  ; age is in yr

   if n_elements(binning) eq 0 and n_elements(defaultbinning) gt 0 then binning = defaultbinning
   if n_elements(resolution) eq 0 and n_elements(defaultresolution) gt 0 then resolution = defaultresolution

   ; check if we need to reload (haven't loaded, or binning/resolution is changed)
   reload = 0
   if n_elements(bc03) eq 0 then reload = 1
   if n_elements(binning) ne n_elements(oldbinning) then reload = 1
   if n_elements(resolution) ne n_elements(resolution) then reload = 1
   if n_elements(binning) gt 0 and n_elements(oldbinning) gt 0 then if binning ne oldbinning then reload = 1
   if n_elements(resolution) gt 0 and n_elements(oldresolution) gt 0 then if resolution ne oldresolution then reload = 1

   if reload then begin
     bc03 = loadbc03(binning=binning, resolution=resolution)

     ; store the SSPs and the binning/resolution values
     if n_elements(binning) gt 0 then oldbinning = binning
     if n_elements(resolution) gt 0 then oldresolution = resolution
     if n_elements(binning) eq 0 and n_elements(oldbinning) gt 0 then delvarx, oldbinning
     if n_elements(resolution) eq 0 and n_elements(oldresolution) gt 0 then delvarx, oldresolution
   endif

   ; set up outputs now in case we have to return early
   nwav = n_elements(bc03.wav)
   Llambda = fltarr(nwav)
   Lyoung = fltarr(nwav) ; this is a kludge to enable age-dependent extinction.
   Lold   = fltarr(nwav) ; in the future it may be better to return a whole grid of L(age,wav)
   wavAng = bc03.wav
   bct = bc03.age

   if n_elements(mass) eq 1 then if mass eq 0 then begin
     ; trivial case
     return, Llambda
   endif

   if n_elements(sfhist) eq 0 then begin
     print, 'Need to specify star-formation history using popsynhist.'
     return, !values.f_nan
   endif

     
     ;if n_elements(age) eq 0. then age = mass / sfr
   ;endif
   ;if n_elements(sfhist) eq 0 and n_elements(tau) gt 0 then begin
;
;   endif

   ; set the age gridding and bin widths (?)
   nage = n_elements(bc03.age)
   lage = (shift(bc03.age, 1) + bc03.age)/2.
   uage = (shift(bc03.age,-1) + bc03.age)/2.
   lage[0] = 0.
   uage[nage-1] = max(bc03.age)*1.2
   dage = (uage - lage)

   ; normalize
   ; sfhist is in Msun/yr, we want Msun/bin.
   if n_elements(mass) gt 0 then begin
     wsfhist = sfhist * dage          ; adjust for binning, now Msun/bin not Msun/yr
     wsfhist /= total(wsfhist)        ; re-normalize to 1 Msun total produced
     wsfhist *= mass
   endif else begin
     wsfhist = sfhist / sfhist[0]  ; renormalize to 1 Msun/year at t=0
     wsfhist *= sfr
     wsfhist *= dage           ; adjust for binning, now Msun/bin
     mass = total(wsfhist)
   endelse

   negsfr = where(wsfhist lt 0, ctneg)
   if ctneg gt 0 then begin
     print, 'WARNING - ', ctneg, ' negative SFR bins detected!  This is not physical; something is wrong.'
   endif

   ; interpolate the metallicity

   nmetal = n_elements(bc03.metal)
   metalweight = bc03.metal * 0.
   metaldiff = bc03.metal - metallicity ; doing this linearly for now.

   nearestm = where(abs(metaldiff) eq min(abs(metaldiff)))
   if metaldiff[nearestm] lt 0 then begin
     nearestmminus = nearestm
     nearestmplus = nearestm+1
   endif
   if metaldiff[nearestm] gt 0 then begin
     nearestmminus = nearestm-1
     nearestmplus = nearestm
   endif
   if metaldiff[nearestm] eq 0 then begin
      metalweight[nearestm] = 1.0
   endif else begin
      metalweight[nearestmplus]  = abs(metaldiff[nearestmminus]) /  (abs(metaldiff[nearestmminus]) + abs(metaldiff[nearestmplus]))
      metalweight[nearestmminus] = 1 -  metalweight[nearestmplus]
   endelse

   ;plot, bc03.metal, metalweight, xrange=[0,3], /xstyle, yrange=[0,1.1], psym=10


   ; Make the SEDs!

   if n_elements(cutage) eq 0 then cutage = -1

   for m = 0, nmetal-1 do begin
     for a = 0, nage-1 do begin
        dL = metalweight[m] * wsfhist[a] * bc03.Llambda[m,a,*]
        if bc03.age[a] lt cutage then Lyoung += dL else Lold += dL
     endfor
   endfor

   Llambda = Lyoung + Lold

   return, Llambda

end



